#!/usr/bin/perl
#
# DESCRIPTION:
#   script to provide external authentication via LDAP server (or
#   Active Directory)
#
# DESIGN:
#   the script uses userid and password supplied on the command line to 
#   connect to specified ldap server and attempts to login (bind). upon
#   upon succesful authentication, the script will exit with return code
#   zero (0) indicating successful authentication. a non-zero value is
#   returned if the login credentials were not accepted.
#
# REQUIRES:
#   Net::LDAP (http://search.cpan.org/search?query=Net::LDAP)
#   Getopt::Long (http://search.cpan.org/search?dist=Getopt-Long)
#  

use strict;
use vars qw($ad_user $ad_pass $ad_server $ad_domain $ad 
            $msg $DEBUG $help $my_name $my_version);

use File::Basename;
use Net::LDAP;
use Net::LDAP::Util;
use Getopt::Long;

## vars --------------------------------------------------
##

$my_name = basename($0);
$my_version = ".01";

GetOptions('user|u=s'     => \$ad_user,
           'password|p=s' => \$ad_pass,
           'server|s=s'   => \$ad_server,
           'domain|d=s'   => \$ad_domain,
           'debug'        => \$DEBUG,
           'help|h'       => \$help);

my $fq_user;
$fq_user = $ad_user . '@' . $ad_domain;

if ($help) {
  show_usage();
}

if ((!$ad_user) || (!$ad_pass) || (!$ad_server) || (!$ad_domain)) {
  print "ERROR: Missing required option user, password, or AD server\n";
  debug_output();
  show_usage();
  exit(2);
}

$ad = Net::LDAP->new($ad_server, debug => $DEBUG) || die $@;
$msg = $ad->bind($fq_user, password=>$ad_pass, version=>3);

if ($msg->code) {
  print "ERROR: Unable to authenticate user " . $fq_user . "\n";
  print "ERROR: " . $msg->error . "\n";
  $ad->unbind;
  debug_output();
  exit(1);
}

else {
  print "user " . $ad_user . " authenticated successfully\n"
}

$ad->unbind;
exit(0);

sub debug_output(){
 if ($DEBUG){
   print "DEBUG:\n";
   print "  user  : " . $ad_user . "\n";
   print "  pass  : " . $ad_pass . "\n";
   print "  domain: " . $ad_domain . "\n";
   print "  server: " . $ad_server . "\n";
   print "\n";
 }
}

sub show_usage {

  print <<eofUsage

  ${my_name} (${my_version})
  usage: ${my_name} --server=<fqdn | ip_address>
         [ --domain=<ad_domain> ] [ --user=<ad_user> ]
         [ --password=<ad_password> ] [--debug] [ --help ]

  --server    = host name or ip address of Active Directory server
  --domain    = domain to authenticate user against
  --user      = user to authenticate
  --password  = password of user
  --debug     = print debug output on errors
  --help      = print this help message

eofUsage
;

  exit(0);

}
